@@ -1,7 +1,7 @@
 /* 
  *  Squeezelite - lightweight headless squeezebox emulator
  *
- *  (c) Adrian Smith 2012, 2013, triode1@btinternet.com
+ *  (c) Adrian Smith 2012-2014, triode1@btinternet.com
  *  
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,22 @@
 
 #include <signal.h>
 
-#define TITLE "Squeezelite " VERSION ", Copyright 2012, 2013 Adrian Smith."
+#define TITLE "Squeezelite " VERSION ", Copyright 2012-2014 Adrian Smith."
+
+#define CODECS_BASE "flac,pcm,mp3,ogg,aac"
+#if FFMPEG
+#define CODECS_FF   ",wma,alac"
+#else
+#define CODECS_FF   ""
+#endif
+#if DSD
+#define CODECS_DSD  ",dsd"
+#else
+#define CODECS_DSD  ""
+#endif
+#define CODECS_MP3  " (mad,mpg for specific mp3 codec)"
+
+#define CODECS CODECS_BASE CODECS_FF CODECS_DSD CODECS_MP3
 
 static void usage(const char *argv0) {
 	printf(TITLE " See -t for license terms\n"
@@ -42,21 +57,17 @@ static void usage(const char *argv0) {
 #endif
 		   "  -a <f>\t\tSpecify sample format (16|24|32) of output file when using -o - to output samples to stdout (interleaved little endian only)\n"
 		   "  -b <stream>:<output>\tSpecify internal Stream and Output buffer sizes in Kbytes\n"
-		   "  -c <codec1>,<codec2>\tRestrict codecs to those specified, otherwise load all available codecs; known codecs: "
-#if FFMPEG
-		   "flac,pcm,mp3,ogg,aac,wma,alac (mad,mpg for specific mp3 codec)\n"
-#else
-		   "flac,pcm,mp3,ogg,aac (mad,mpg for specific mp3 codec)\n"
-#endif
+		   "  -c <codec1>,<codec2>\tRestrict codecs to those specified, otherwise load all available codecs; known codecs: " CODECS "\n"
 		   "  -d <log>=<level>\tSet logging level, logs: all|slimproto|stream|decode|output, level: info|debug|sdebug\n"
+		   "  -e <codec1>,<codec2>\tExplicitly exclude native support of one or more codecs; known codecs: " CODECS "\n"
 		   "  -f <logfile>\t\tWrite debug to logfile\n"
 		   "  -m <mac addr>\t\tSet mac address, format: ab:cd:ef:12:34:56\n"
 		   "  -n <name>\t\tSet the player name\n"
-		   "  -N <filename>\t\tStore player name in filename to allow server defined name changes to be shared between servers (not suppored with -n)\n"
+		   "  -N <filename>\t\tStore player name in filename to allow server defined name changes to be shared between servers (not supported with -n)\n"
 #if ALSA
 		   "  -p <priority>\t\tSet real time priority of output thread (1-99)\n"
 #endif
-		   "  -r <rates>\t\tSpecify sample rates supported by device, enables output device to be off when squeezelite is started; rates = <maxrate> | <minrate>-<maxrate> | <rate1>,<rate2>,<rate3>\n"
+		   "  -r <rates>[:<delay>]\tSample rates supported, allows output to be off when squeezelite is started; rates = <maxrate>|<minrate>-<maxrate>|<rate1>,<rate2>,<rate3>; delay = optional delay switching rates in ms\n"
 #if RESAMPLE
 		   "  -R -u [params]\tResample, params = <recipe>:<flags>:<attenuation>:<precision>:<passband_end>:<stopband_start>:<phase_response>,\n" 
 		   "  \t\t\t recipe = (v|h|m|l|q)(L|I|M)(s) [E|X], E = exception - resample only if native rate not supported, X = async - resample to max rate for device, otherwise to max sync rate\n"
@@ -68,12 +79,12 @@ static void usage(const char *argv0) {
 		   "  \t\t\t phase_response = 0-100 (0 = minimum / 50 = linear / 100 = maximum)\n"
 #endif
 #if DSD
-		   "  -D\t\t\tOutput device supports DSD over PCM (DoP)\n" 
+		   "  -D [delay]\t\tOutput device supports DSD over PCM (DoP), delay = optional delay switching between PCM and DoP in ms\n" 
 #endif
 #if VISEXPORT
-		   "  -v \t\t\tVisulizer support\n"
+		   "  -v \t\t\tVisualiser support\n"
 #endif
-#if LINUX
+#if LINUX || FREEBSD
 		   "  -z \t\t\tDaemonize\n"
 #endif
 		   "  -t \t\t\tLicense terms\n"
@@ -88,6 +99,9 @@ static void usage(const char *argv0) {
 #if OSX
 		   " OSX"
 #endif
+#if FREEBSD
+		   " FREEBSD"
+#endif
 #if ALSA
 		   " ALSA"
 #endif
@@ -103,9 +117,13 @@ static void usage(const char *argv0) {
 #if WINEVENT
 		   " WINEVENT"
 #endif
+#if RESAMPLE_MP
+		   " RESAMPLE_MP"
+#else
 #if RESAMPLE
 		   " RESAMPLE"
 #endif
+#endif
 #if FFMPEG
 		   " FFMPEG"
 #endif
@@ -151,7 +169,8 @@ static void sighandler(int signum) {
 int main(int argc, char **argv) {
 	char *server = NULL;
 	char *output_device = "default";
-	char *codecs = NULL;
+	char *include_codecs = NULL;
+	char *exclude_codecs = "";
 	char *name = NULL;
 	char *namefile = NULL;
 	char *logfile = NULL;
@@ -159,9 +178,10 @@ int main(int argc, char **argv) {
 	unsigned stream_buf_size = STREAMBUF_SIZE;
 	unsigned output_buf_size = 0; // set later
 	unsigned rates[MAX_SUPPORTED_SAMPLERATES] = { 0 };
+	unsigned rate_delay = 0;
 	char *resample = NULL;
 	char *output_params = NULL;
-#if LINUX
+#if LINUX || FREEBSD
 	bool daemonize = false;
 #endif
 #if ALSA
@@ -169,6 +189,7 @@ int main(int argc, char **argv) {
 #endif
 #if DSD
 	bool dop = false;
+	unsigned dop_delay = 0;
 #endif
 #if VISEXPORT
 	bool visexport = false;
@@ -181,12 +202,21 @@ int main(int argc, char **argv) {
 
 	char *optarg = NULL;
 	int optind = 1;
+	int i;
+
+#define MAXCMDLINE 256
+	char cmdline[MAXCMDLINE] = "";
 
 	get_mac(mac);
 
+	for (i = 0; i < argc && (strlen(argv[i]) + strlen(cmdline) + 2 < MAXCMDLINE); i++) {
+		strcat(cmdline, argv[i]);
+		strcat(cmdline, " ");
+	}
+
 	while (optind < argc && strlen(argv[optind]) >= 2 && argv[optind][0] == '-') {
 		char *opt = argv[optind] + 1;
-		if (strstr("oabcdfmnNprs", opt) && optind < argc - 1) {
+		if (strstr("oabcdefmnNprs", opt) && optind < argc - 1) {
 			optarg = argv[optind + 1];
 			optind += 2;
 		} else if (strstr("ltz"
@@ -223,7 +253,10 @@ int main(int argc, char **argv) {
 			}
 			break;
 		case 'c':
-			codecs = optarg;
+			include_codecs = optarg;
+			break;
+		case 'e':
+			exclude_codecs = optarg;
 			break;
         case 'd':
 			{
@@ -259,40 +292,47 @@ int main(int argc, char **argv) {
 			}
 			break;
 		case 'r':
-			if (strstr(optarg,",")) {
-				// parse sample rates and sort them
-				char *r = next_param(optarg, ',');
-				unsigned tmp[MAX_SUPPORTED_SAMPLERATES] = { 0 };
-				int i, j;
-				int last = 999999;
-				for (i = 0; r && i < MAX_SUPPORTED_SAMPLERATES; ++i) { 
-					tmp[i] = atoi(r);
-					r = next_param(NULL, ',');
-				}
-				for (i = 0; i < MAX_SUPPORTED_SAMPLERATES; ++i) {
-					int largest = 0;
-					for (j = 0; j < MAX_SUPPORTED_SAMPLERATES; ++j) {
-						if (tmp[j] > largest && tmp[j] < last) {
-							largest = tmp[j];
+			{ 
+				char *rstr = next_param(optarg, ':');
+				char *dstr = next_param(NULL, ':');
+				if (rstr && strstr(rstr, ",")) {
+					// parse sample rates and sort them
+					char *r = next_param(rstr, ',');
+					unsigned tmp[MAX_SUPPORTED_SAMPLERATES] = { 0 };
+					int i, j;
+					int last = 999999;
+					for (i = 0; r && i < MAX_SUPPORTED_SAMPLERATES; ++i) { 
+						tmp[i] = atoi(r);
+						r = next_param(NULL, ',');
+					}
+					for (i = 0; i < MAX_SUPPORTED_SAMPLERATES; ++i) {
+						int largest = 0;
+						for (j = 0; j < MAX_SUPPORTED_SAMPLERATES; ++j) {
+							if (tmp[j] > largest && tmp[j] < last) {
+								largest = tmp[j];
+							}
+						}
+						rates[i] = last = largest;
+					}
+				} else if (rstr) {
+					// optstr is <min>-<max> or <max>, extract rates from test rates within this range
+					unsigned ref[] TEST_RATES;
+					char *str1 = next_param(rstr, '-');
+					char *str2 = next_param(NULL, '-');
+					unsigned max = str2 ? atoi(str2) : (str1 ? atoi(str1) : ref[0]);
+					unsigned min = str1 && str2 ? atoi(str1) : 0;
+					unsigned tmp;
+					int i, j;
+					if (max < min) { tmp = max; max = min; min = tmp; }
+					rates[0] = max;
+					for (i = 0, j = 1; i < MAX_SUPPORTED_SAMPLERATES; ++i) {
+						if (ref[i] < rates[j-1] && ref[i] >= min) {
+							rates[j++] = ref[i];
 						}
 					}
-					rates[i] = last = largest;
 				}
-			} else {
-				// optstr is <min>-<max> or <max>, extract rates from test rates within this range
-				unsigned ref[] TEST_RATES;
-				char *str1 = next_param(optarg, '-');
-				char *str2 = next_param(NULL, '-');
-				unsigned max = str2 ? atoi(str2) : (str1 ? atoi(str1) : ref[0]);
-				unsigned min = str1 && str2 ? atoi(str1) : 0;
-				unsigned tmp;
-				int i, j;
-				if (max < min) { tmp = max; max = min; min = tmp; }
-				rates[0] = max;
-				for (i = 0, j = 1; i < MAX_SUPPORTED_SAMPLERATES; ++i) {
-					if (ref[i] < rates[j-1] && ref[i] >= min) {
-						rates[j++] = ref[i];
-					}
+				if (dstr) {
+					rate_delay = atoi(dstr);
 				}
 			}
 			break;
@@ -331,6 +371,9 @@ int main(int argc, char **argv) {
 #if DSD
 		case 'D':
 			dop = true;
+			if (optind < argc && argv[optind] && argv[optind][0] != '-') {
+				dop_delay = atoi(argv[optind++]);
+			}
 			break;
 #endif
 #if VISEXPORT
@@ -338,7 +381,7 @@ int main(int argc, char **argv) {
 			visexport = true;
 			break;
 #endif
-#if LINUX
+#if LINUX || FREEBSD
 		case 'z':
 			daemonize = true;
 			break;
@@ -351,6 +394,12 @@ int main(int argc, char **argv) {
         }
     }
 
+	// warn if command line includes something which isn't parsed
+	if (optind < argc) {
+		usage(argv[0]);
+		exit(0);
+	}
+
 	signal(SIGINT, sighandler);
 	signal(SIGTERM, sighandler);
 #if defined(SIGQUIT)
@@ -375,12 +424,16 @@ int main(int argc, char **argv) {
 	}
 
 	if (logfile) {
-		if (!freopen(logfile, "a", stdout) || !freopen(logfile, "a", stderr)) {
+		if (!freopen(logfile, "a", stderr)) {
 			fprintf(stderr, "error opening logfile %s: %s\n", logfile, strerror(errno));
+		} else {
+			if (log_output >= lINFO || log_stream >= lINFO || log_decode >= lINFO || log_slimproto >= lINFO) {
+				fprintf(stderr, "\n%s\n", cmdline);
+			}
 		}
 	}
 
-#if LINUX
+#if LINUX || FREEBSD
 	if (daemonize) {
 		if (daemon(0, logfile ? 1 : 0)) {
 			fprintf(stderr, "error daemonizing: %s\n", strerror(errno));
@@ -395,18 +448,18 @@ int main(int argc, char **argv) {
 	stream_init(log_stream, stream_buf_size);
 
 	if (!strcmp(output_device, "-")) {
-		output_init_stdout(log_output, output_buf_size, output_params, rates);
+		output_init_stdout(log_output, output_buf_size, output_params, rates, rate_delay);
 	} else {
 #if ALSA
-		output_init_alsa(log_output, output_device, output_buf_size, output_params, rates, rt_priority);
+		output_init_alsa(log_output, output_device, output_buf_size, output_params, rates, rate_delay, rt_priority);
 #endif
 #if PORTAUDIO
-		output_init_pa(log_output, output_device, output_buf_size, output_params, rates);
+		output_init_pa(log_output, output_device, output_buf_size, output_params, rates, rate_delay);
 #endif
 	}
 
 #if DSD
-	dop_init(dop);
+	dop_init(dop, dop_delay);
 #endif
 
 #if VISEXPORT
@@ -415,7 +468,7 @@ int main(int argc, char **argv) {
 	}
 #endif
 
-	decode_init(log_decode, codecs);
+	decode_init(log_decode, include_codecs, exclude_codecs);
 
 #if RESAMPLE
 	if (resample) {
