@@ -1,7 +1,7 @@
 /* 
  *  Squeezelite - lightweight headless squeezebox emulator
  *
- *  (c) Adrian Smith 2012, 2013, triode1@btinternet.com
+ *  (c) Adrian Smith 2012-2014, triode1@btinternet.com
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -532,6 +532,7 @@ static void *output_thread(void *arg) {
 							continue;
 						}
 						LOG_INFO("start error: %s", snd_strerror(err));
+						usleep(10000);
 					}
 				} else {
 					start = false;
@@ -579,18 +580,31 @@ static void *output_thread(void *arg) {
 				// EPIPE indicates underrun - attempt to recover
 				UNLOCK;
 				continue;
+			} else if (err == -EIO) {
+				// EIO can occur with non existant pulse server
+				UNLOCK;
+				LOG_SDEBUG("snd_pcm_delay returns: EIO - sleeping");
+				usleep(100000);
+				continue;
 			} else {
 				LOG_DEBUG("snd_pcm_delay returns: %d", err);
 			}
 		} else {
 			output.device_frames = delay;
 			output.updated = gettime_ms();
+			output.frames_played_dmp = output.frames_played;
 		}
 
 		// process frames
-		_output_frames(avail);
+		frames_t wrote = _output_frames(avail);
 
 		UNLOCK;
+
+		// some output devices such as alsa null refuse any data, avoid spinning
+		if (!wrote) {
+			LOG_SDEBUG("wrote 0 - sleeping");
+			usleep(10000);
+		}
 	}
 
 	return 0;
@@ -598,7 +612,8 @@ static void *output_thread(void *arg) {
 
 static pthread_t thread;
 
-void output_init_alsa(log_level level, const char *device, unsigned output_buf_size, char *params, unsigned rates[], unsigned rt_priority) {
+void output_init_alsa(log_level level, const char *device, unsigned output_buf_size, char *params, unsigned rates[], 
+					  unsigned rate_delay, unsigned rt_priority) {
 
 	unsigned alsa_buffer = ALSA_BUFFER_TIME;
 	unsigned alsa_period = ALSA_PERIOD_COUNT;
@@ -633,6 +648,7 @@ void output_init_alsa(log_level level, const char *device, unsigned output_buf_s
 	output.period = alsa_period;
 	output.start_frames = 0;
 	output.write_cb = &_write_frames;
+	output.rate_delay = rate_delay;
 
 	if (alsa_sample_fmt) {
 		if (!strcmp(alsa_sample_fmt, "32"))	alsa.format = SND_PCM_FORMAT_S32_LE;
