@@ -1,7 +1,7 @@
 /* 
  *  Squeezelite - lightweight headless squeezebox emulator
  *
- *  (c) Adrian Smith 2012, 2013, triode1@btinternet.com
+ *  (c) Adrian Smith 2012-2014, triode1@btinternet.com
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -379,6 +379,6 @@ struct codec *register_pcm(void) {
 		pcm_decode,  // decode
 	};
 
-	LOG_INFO("using pcm");
+	LOG_INFO("using pcm to decode aif,pcm");
 	return &ret;
 }
