@@ -1,7 +1,7 @@
 /* 
  *  Squeezelite - lightweight headless squeezebox emulator
  *
- *  (c) Adrian Smith 2012, 2013, triode1@btinternet.com
+ *  (c) Adrian Smith 2012-2014, triode1@btinternet.com
  *  
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -116,7 +116,7 @@ static void sendHELO(bool reconnect, const char *fixed_cap, const char *var_cap,
 
 	LOG_INFO("mac: %02x:%02x:%02x:%02x:%02x:%02x", pkt.mac[0], pkt.mac[1], pkt.mac[2], pkt.mac[3], pkt.mac[4], pkt.mac[5]);
 
-	LOG_DEBUG("cap: %s%s%s", base_cap, fixed_cap, var_cap);
+	LOG_INFO("cap: %s%s%s", base_cap, fixed_cap, var_cap);
 
 	send_packet((u8_t *)&pkt, sizeof(pkt));
 	send_packet((u8_t *)base_cap, strlen(base_cap));
@@ -132,7 +132,12 @@ static void sendSTAT(const char *event, u32_t server_timestamp) {
 	if (status.current_sample_rate && status.frames_played && status.frames_played > status.device_frames) {
 		ms_played = (u32_t)(((u64_t)(status.frames_played - status.device_frames) * (u64_t)1000) / (u64_t)status.current_sample_rate);
 		if (now > status.updated) ms_played += (now - status.updated);
+		LOG_SDEBUG("ms_played: %u (frames_played: %u device_frames: %u)", ms_played, status.frames_played, status.device_frames);
+	} else if (status.frames_played && now > status.stream_start) {
+		ms_played = now - status.stream_start;
+		LOG_SDEBUG("ms_played: %u using elapsed time (frames_played: %u device_frames: %u)", ms_played, status.frames_played, status.device_frames);
 	} else {
+		LOG_SDEBUG("ms_played: 0");
 		ms_played = 0;
 	}
 	
@@ -156,10 +161,10 @@ static void sendSTAT(const char *event, u32_t server_timestamp) {
 	pkt.server_timestamp = server_timestamp; // keep this is server format - don't unpack/pack
 	// error_code;
 
-	LOG_INFO("STAT: %s", event);
+	LOG_DEBUG("STAT: %s", event);
 
 	if (loglevel == lSDEBUG) {
-		LOG_SDEBUG("received bytesL: %u streambuf: %u outputbuf: %u calc elapsed: %u real elapsed: %u (diff: %u) device: %u delay: %d",
+		LOG_SDEBUG("received bytesL: %u streambuf: %u outputbuf: %u calc elapsed: %u real elapsed: %u (diff: %d) device: %u delay: %d",
 				   (u32_t)status.stream_bytes, status.stream_full, status.output_full, ms_played, now - status.stream_start,
 				   ms_played - now + status.stream_start, status.device_frames * 1000 / status.current_sample_rate, now - status.updated);
 	}
@@ -175,7 +180,7 @@ static void sendDSCO(disconnect_code disconnect) {
 	pkt.length = htonl(sizeof(pkt) - 8);
 	pkt.reason = disconnect & 0xFF;
 
-	LOG_INFO("DSCO: %d", disconnect);
+	LOG_DEBUG("DSCO: %d", disconnect);
 
 	send_packet((u8_t *)&pkt, sizeof(pkt));
 }
@@ -187,7 +192,7 @@ static void sendRESP(const char *header, size_t len) {
 	memcpy(&pkt_header.opcode, "RESP", 4);
 	pkt_header.length = htonl(sizeof(pkt_header) + len - 8);
 
-	LOG_INFO("RESP");
+	LOG_DEBUG("RESP");
 
 	send_packet((u8_t *)&pkt_header, sizeof(pkt_header));
 	send_packet((u8_t *)header, len);
@@ -200,7 +205,7 @@ static void sendMETA(const char *meta, size_t len) {
 	memcpy(&pkt_header.opcode, "META", 4);
 	pkt_header.length = htonl(sizeof(pkt_header) + len - 8);
 
-	LOG_INFO("META");
+	LOG_DEBUG("META");
 
 	send_packet((u8_t *)&pkt_header, sizeof(pkt_header));
 	send_packet((u8_t *)meta, len);
@@ -215,7 +220,7 @@ static void sendSETDName(const char *name) {
 	pkt_header.id = 0; // id 0 is playername S:P:Squeezebox2
 	pkt_header.length = htonl(sizeof(pkt_header) + strlen(name) + 1 - 8);
 
-	LOG_INFO("set playername: %s", name);
+	LOG_DEBUG("set playername: %s", name);
 
 	send_packet((u8_t *)&pkt_header, sizeof(pkt_header));
 	send_packet((u8_t *)name, strlen(name) + 1);
@@ -224,7 +229,7 @@ static void sendSETDName(const char *name) {
 static void process_strm(u8_t *pkt, int len) {
 	struct strm_packet *strm = (struct strm_packet *)pkt;
 
-	LOG_INFO("strm command %c", strm->command);
+	LOG_DEBUG("strm command %c", strm->command);
 
 	switch(strm->command) {
 	case 't':
@@ -255,7 +260,7 @@ static void process_strm(u8_t *pkt, int len) {
 			output.state = interval ? OUTPUT_PAUSE_FRAMES : OUTPUT_STOPPED;				
 			UNLOCK_O;
 			if (!interval) sendSTAT("STMp", 0);
-			LOG_INFO("pause interval: %u", interval);
+			LOG_DEBUG("pause interval: %u", interval);
 		}
 		break;
 	case 'a':
@@ -265,7 +270,7 @@ static void process_strm(u8_t *pkt, int len) {
 			output.skip_frames = interval * status.current_sample_rate / 1000;
 			output.state = OUTPUT_SKIP_FRAMES;				
 			UNLOCK_O;
-			LOG_INFO("skip ahead interval: %u", interval);
+			LOG_DEBUG("skip ahead interval: %u", interval);
 		}
 		break;
 	case 'u':
@@ -278,7 +283,7 @@ static void process_strm(u8_t *pkt, int len) {
 			LOCK_D;
 			decode.state = DECODE_RUNNING;
 			UNLOCK_D;
-			LOG_INFO("unpause at: %u now: %u", jiffies, gettime_ms());
+			LOG_DEBUG("unpause at: %u now: %u", jiffies, gettime_ms());
 			sendSTAT("STMr", 0);
 		}
 		break;
@@ -290,9 +295,9 @@ static void process_strm(u8_t *pkt, int len) {
 			u16_t port = strm->server_port; // keep in network byte order
 			if (ip == 0) ip = slimproto_ip; 
 
-			LOG_INFO("strm s autostart: %c transition period: %u transition type: %u codec: %c", 
-					 strm->autostart, strm->transition_period, strm->transition_type - '0', strm->format);
-
+			LOG_DEBUG("strm s autostart: %c transition period: %u transition type: %u codec: %c", 
+					  strm->autostart, strm->transition_period, strm->transition_type - '0', strm->format);
+			
 			autostart = strm->autostart - '0';
 			sendSTAT("STMf", 0);
 			if (header_len > MAX_HEADER -1) {
@@ -303,7 +308,7 @@ static void process_strm(u8_t *pkt, int len) {
 				codec_open(strm->format, strm->pcm_sample_size, strm->pcm_sample_rate, strm->pcm_channels, strm->pcm_endianness);
 			} else if (autostart >= 2) {
 				// extension to slimproto to allow server to detect codec from response header and send back in codc message
-				LOG_INFO("streaming unknown codec");
+				LOG_DEBUG("streaming unknown codec");
 			} else {
 				LOG_WARN("unknown codec requires autostart >= 2");
 				break;
@@ -322,12 +327,12 @@ static void process_strm(u8_t *pkt, int len) {
 			output.next_replay_gain = unpackN(&strm->replay_gain);
 			output.fade_mode = strm->transition_type - '0';
 			output.fade_secs = strm->transition_period;
-			LOG_INFO("set fade mode: %u", output.fade_mode);
+			LOG_DEBUG("set fade mode: %u", output.fade_mode);
 			UNLOCK_O;
 		}
 		break;
 	default:
-		LOG_INFO("unhandled strm %c", strm->command);
+		LOG_WARN("unhandled strm %c", strm->command);
 		break;
 	}
 }
@@ -336,7 +341,7 @@ static void process_cont(u8_t *pkt, int len) {
 	struct cont_packet *cont = (struct cont_packet *)pkt;
 	cont->metaint = unpackN(&cont->metaint);
 
-	LOG_INFO("cont metaint: %u loop: %u", cont->metaint, cont->loop);
+	LOG_DEBUG("cont metaint: %u loop: %u", cont->metaint, cont->loop);
 
 	if (autostart > 1) {
 		autostart -= 2;
@@ -353,14 +358,14 @@ static void process_cont(u8_t *pkt, int len) {
 static void process_codc(u8_t *pkt, int len) {
 	struct codc_packet *codc = (struct codc_packet *)pkt;
 
-	LOG_INFO("codc: %c", codc->format);
+	LOG_DEBUG("codc: %c", codc->format);
 	codec_open(codc->format, codc->pcm_sample_size, codc->pcm_sample_rate, codc->pcm_channels, codc->pcm_endianness);
 }
 
 static void process_aude(u8_t *pkt, int len) {
 	struct aude_packet *aude = (struct aude_packet *)pkt;
 
-	LOG_INFO("enable spdif: %d dac: %d", aude->enable_spdif, aude->enable_dac);
+	LOG_DEBUG("enable spdif: %d dac: %d", aude->enable_spdif, aude->enable_dac);
 
 	LOCK_O;
 	if (!aude->enable_spdif && output.state != OUTPUT_OFF) {
@@ -377,7 +382,7 @@ static void process_audg(u8_t *pkt, int len) {
 	audg->gainL = unpackN(&audg->gainL);
 	audg->gainR = unpackN(&audg->gainR);
 
-	LOG_INFO("audg gainL: %u gainR: %u adjust: %u", audg->gainL, audg->gainR, audg->adjust);
+	LOG_DEBUG("audg gainL: %u gainR: %u adjust: %u", audg->gainL, audg->gainR, audg->adjust);
 
 	LOCK_O;
 	output.gainL = audg->adjust ? audg->gainL : FIXED_ONE;
@@ -461,11 +466,11 @@ static void process(u8_t *pack, int len) {
 	while (h->handler && strncmp((char *)pack, h->opcode, 4)) { h++; }
 
 	if (h->handler) {
-		LOG_INFO("%s", h->opcode);
+		LOG_DEBUG("%s", h->opcode);
 		h->handler(pack, len);
 	} else {
 		pack[4] = '\0';
-		LOG_INFO("unhandled %s", (char *)pack);
+		LOG_WARN("unhandled %s", (char *)pack);
 	}
 }
 
@@ -558,7 +563,8 @@ static void slimproto_run() {
 			bool _sendSTMu = false;
 			bool _sendSTMo = false;
 			bool _sendSTMn = false;
-			disconnect_code disconnect;
+			bool _stream_disconnect = false;
+			disconnect_code disconnect_code;
 			static char header[MAX_HEADER];
 			size_t header_len = 0;
 			last = now;
@@ -570,7 +576,7 @@ static void slimproto_run() {
 			status.stream_state = stream.state;
 						
 			if (stream.state == DISCONNECT) {
-				disconnect = stream.disconnect;
+				disconnect_code = stream.disconnect;
 				stream.state = STOPPED;
 				_sendDSCO = true;
 			}
@@ -592,7 +598,7 @@ static void slimproto_run() {
 			LOCK_O;
 			status.output_full = _buf_used(outputbuf);
 			status.output_size = outputbuf->size;
-			status.frames_played = output.frames_played;
+			status.frames_played = output.frames_played_dmp;
 			status.current_sample_rate = output.current_sample_rate;
 			status.updated = output.updated;
 			status.device_frames = output.device_frames;
@@ -600,7 +606,7 @@ static void slimproto_run() {
 			if (output.track_started) {
 				_sendSTMs = true;
 				output.track_started = false;
-				status.stream_start = output.updated;
+				status.stream_start = output.track_start_time;
 			}
 #if PORTAUDIO
 			if (output.pa_reopen) {
@@ -623,14 +629,6 @@ static void slimproto_run() {
 				_sendSTMt = true;
 				status.last = now;
 			}
-			if (decode.state == DECODE_COMPLETE) {
-				_sendSTMd = true;
-				decode.state = DECODE_STOPPED;
-			}
-			if (decode.state == DECODE_ERROR) {
-				_sendSTMn = true;
-				decode.state = DECODE_STOPPED;
-			}
 			if ((status.stream_state == STREAMING_HTTP || status.stream_state == STREAMING_FILE) && !sentSTMl 
 				&& decode.state == DECODE_STOPPED) {
 				if (autostart == 0) {
@@ -646,10 +644,20 @@ static void slimproto_run() {
 				}
 				// autostart 2 and 3 require cont to be received first
 			}
+			if (decode.state == DECODE_COMPLETE || decode.state == DECODE_ERROR) {
+				if (decode.state == DECODE_COMPLETE) _sendSTMd = true;
+				if (decode.state == DECODE_ERROR)    _sendSTMn = true;
+				decode.state = DECODE_STOPPED;
+				if (status.stream_state == STREAMING_HTTP || status.stream_state == STREAMING_FILE) { 
+					_stream_disconnect = true;
+				}
+			}
 			UNLOCK_D;
 		
+			if (_stream_disconnect) stream_disconnect();
+
 			// send packets once locks released as packet sending can block
-			if (_sendDSCO) sendDSCO(disconnect);
+			if (_sendDSCO) sendDSCO(disconnect_code);
 			if (_sendSTMs) sendSTAT("STMs", 0);
 			if (_sendSTMd) sendSTAT("STMd", 0);
 			if (_sendSTMt) sendSTAT("STMt", 0);
@@ -750,7 +758,12 @@ void slimproto(log_level level, char *server, u8_t mac[6], const char *name, con
 			if (!fgets(player_name, PLAYER_NAME_LEN, fp)) {
 				player_name[PLAYER_NAME_LEN] = '\0';
 			} else {
-				LOG_INFO("retrived name %s from %s", player_name, name_file);
+				// strip any \n from fgets response
+				int len = strlen(player_name);
+				if (len > 0 && player_name[len - 1] == '\n') {
+					player_name[len - 1] = '\0';
+				}
+				LOG_INFO("retrieved name %s from %s", player_name, name_file);
 			}
 			fclose(fp);
 		}
@@ -804,18 +817,16 @@ void slimproto(log_level level, char *server, u8_t mac[6], const char *name, con
 
 		} else {
 
+			struct sockaddr_in our_addr;
+			socklen_t len;
+
 			LOG_INFO("connected");
 
 			var_cap[0] = '\0';
-
 			failed_connect = 0;
 
-#if !WIN
 			// check if this is a local player now we are connected & signal to server via 'loc' format
 			// this requires LocalPlayer server plugin to enable direct file access
-			// not supported on windows at present as the poll in stream.c does not work for file access
-			struct sockaddr_in our_addr;
-			socklen_t len;
 			len = sizeof(our_addr);
 			getsockname(sock, (struct sockaddr *) &our_addr, &len);
 
@@ -823,7 +834,6 @@ void slimproto(log_level level, char *server, u8_t mac[6], const char *name, con
 				LOG_INFO("local player");
 				strcat(var_cap, ",loc");
 			}
-#endif
 
 			// add on any capablity to be sent to the new server
 			if (new_server_cap) {
