@@ -1,7 +1,7 @@
 /* 
  *  Squeezelite - lightweight headless squeezebox emulator
  *
- *  (c) Adrian Smith 2012, 2013, triode1@btinternet.com
+ *  (c) Adrian Smith 2012-2014, triode1@btinternet.com
  *  
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,23 +18,31 @@
  *
  */
 
-// make may define: PORTAUDIO, SELFPIPE, RESAMPLE, VISEXPORT, DSD, LINKALL to influence build
+// make may define: PORTAUDIO, SELFPIPE, RESAMPLE, RESAMPLE_MP, VISEXPORT, DSD, LINKALL to influence build
 
-#define VERSION "v1.4"
+#define VERSION "v1.6.4"
 
 // build detection
 #if defined(linux)
 #define LINUX     1
 #define OSX       0
 #define WIN       0
+#define FREEBSD   0
 #elif defined (__APPLE__)
 #define LINUX     0
 #define OSX       1
 #define WIN       0
+#define FREEBSD   0
 #elif defined (_MSC_VER)
 #define LINUX     0
 #define OSX       0
 #define WIN       1
+#define FREEBSD   0
+#elif defined(__FreeBSD__)
+#define LINUX     0
+#define OSX       0
+#define WIN       0
+#define FREEBSD   1
 #else
 #error unknown target
 #endif
@@ -52,7 +60,7 @@
 #define SELFPIPE  0
 #define WINEVENT  0
 #endif
-#if (LINUX && !EVENTFD) || OSX
+#if (LINUX && !EVENTFD) || OSX || FREEBSD
 #define EVENTFD   0
 #define SELFPIPE  1
 #define WINEVENT  0
@@ -63,7 +71,7 @@
 #define WINEVENT  1
 #endif
 
-#if defined(RESAMPLE)
+#if defined(RESAMPLE) || defined(RESAMPLE_MP)
 #undef  RESAMPLE
 #define RESAMPLE  1 // resampling
 #define PROCESS   1 // any sample processing (only resampling at present)
@@ -71,6 +79,12 @@
 #define RESAMPLE  0
 #define PROCESS   0
 #endif
+#if defined(RESAMPLE_MP)
+#undef RESAMPLE_MP
+#define RESAMPLE_MP 1
+#else
+#define RESAMPLE_MP 0
+#endif
 
 #if defined(FFMPEG)
 #undef FFMPEG
@@ -147,6 +161,18 @@
 #define LIBSOXR "libsoxr.dll"
 #endif
 
+#if FREEBSD
+#define LIBFLAC "libFLAC.so.11"
+#define LIBMAD  "libmad.so.2"
+#define LIBMPG "libmpg123.so.0"
+#define LIBVORBIS "libvorbisfile.so.6"
+#define LIBTREMOR "libvorbisidec.so.1"
+#define LIBFAAD "libfaad.so.2"
+#define LIBAVUTIL   "libavutil.so.%d"
+#define LIBAVCODEC  "libavcodec.so.%d"
+#define LIBAVFORMAT "libavformat.so.%d"
+#endif
+
 #endif // !LINKALL
 
 // config options
@@ -172,7 +198,7 @@
 #include <limits.h>
 #include <sys/types.h>
 
-#if LINUX || OSX
+#if LINUX || OSX || FREEBSD
 #include <unistd.h>
 #include <stdbool.h>
 #include <netinet/in.h>
@@ -296,7 +322,7 @@ struct wake {
 #endif
 
 // printf/scanf formats for u64_t
-#if LINUX && __WORDSIZE == 64
+#if (LINUX && __WORDSIZE == 64) || (FREEBSD && __LP64__)
 #define FMT_u64 "%lu"
 #define FMT_x64 "%lx"
 #elif __GLIBC_HAVE_LONG_LONG || defined __GNUC__ || WIN
@@ -354,7 +380,7 @@ void *dlsym(void *handle, const char *symbol);
 char *dlerror(void);
 int poll(struct pollfd *fds, unsigned long numfds, int timeout);
 #endif
-#if LINUX
+#if LINUX || FREEBSD
 void touch_memory(u8_t *buf, size_t size);
 #endif
 
@@ -446,7 +472,7 @@ struct codec {
 	decode_state (*decode)(void);
 };
 
-void decode_init(log_level level, const char *opt);
+void decode_init(log_level level, const char *include_codecs, const char *exclude_codecs);
 void decode_close(void);
 void decode_flush(void);
 unsigned decode_newstream(unsigned sample_rate, unsigned supported_rates[]);
@@ -500,12 +526,14 @@ struct outputstate {
 	int (* write_cb)(frames_t out_frames, bool silence, s32_t gainL, s32_t gainR, s32_t cross_gain_in, s32_t cross_gain_out, s32_t **cross_ptr);
 	unsigned start_frames;
 	unsigned frames_played;
+	unsigned frames_played_dmp;// frames played at the point delay is measured
 	unsigned current_sample_rate;
 	unsigned supported_rates[MAX_SUPPORTED_SAMPLERATES]; // ordered largest first so [0] is max_rate
 	unsigned default_sample_rate;
 	bool error_opening;
 	unsigned device_frames;
 	u32_t updated;
+	u32_t track_start_time;
 	u32_t current_replay_gain;
 	union {
 		u32_t pause_frames;
@@ -524,10 +552,13 @@ struct outputstate {
 	fade_dir fade_dir;
 	fade_mode fade_mode;       // set by slimproto
 	unsigned fade_secs;        // set by slimproto
+	unsigned rate_delay;
+	bool delay_active;
 #if DSD
 	bool next_dop;             // set in decode thread
 	bool dop;
 	bool has_dop;              // set in dop_init - output device supports dop
+	unsigned dop_delay;        // set in dop_init - delay in ms switching to/from dop
 #endif
 };
 
@@ -542,7 +573,8 @@ void _checkfade(bool);
 #if ALSA
 void list_devices(void);
 bool test_open(const char *device, unsigned rates[]);
-void output_init_alsa(log_level level, const char *device, unsigned output_buf_size, char *params, unsigned rates[], unsigned rt_priority);
+void output_init_alsa(log_level level, const char *device, unsigned output_buf_size, char *params, unsigned rates[], 
+					  unsigned rate_delay, unsigned rt_priority);
 void output_close_alsa(void);
 #endif
 
@@ -550,13 +582,13 @@ void output_close_alsa(void);
 #if PORTAUDIO
 void list_devices(void);
 bool test_open(const char *device, unsigned rates[]);
-void output_init_pa(log_level level, const char *device, unsigned output_buf_size, char *params, unsigned rates[]);
+void output_init_pa(log_level level, const char *device, unsigned output_buf_size, char *params, unsigned rates[], unsigned rate_delay);
 void output_close_pa(void);
 void _pa_open(void);
 #endif
 
 // output_stdout.c
-void output_init_stdout(log_level level, unsigned output_buf_size, char *params, unsigned rates[]);
+void output_init_stdout(log_level level, unsigned output_buf_size, char *params, unsigned rates[], unsigned rate_delay);
 void output_close_stdout(void);
 
 // output_pack.c
@@ -581,7 +613,7 @@ void vis_stop(void);
 bool is_flac_dop(u32_t *lptr, u32_t *rptr, frames_t frames);
 void update_dop_marker(u32_t *ptr, frames_t frames);
 void dop_silence_frames(u32_t *ptr, frames_t frames);
-void dop_init(bool enable);
+void dop_init(bool enable, unsigned delay);
 #endif
 
 // codecs
